@@ -70,6 +70,7 @@ enum VBoxControlUsage
 #ifdef RT_OS_WINDOWS
     GET_VIDEO_ACCEL,
     SET_VIDEO_ACCEL,
+    VIDEO_FLAGS,
     LIST_CUST_MODES,
     ADD_CUST_MODE,
     REMOVE_CUST_MODE,
@@ -107,6 +108,8 @@ static void usage(enum VBoxControlUsage eWhich = USAGE_ALL)
         doUsage("", g_pszProgName, "getvideoacceleration");
     if (SET_VIDEO_ACCEL == eWhich || eWhich == USAGE_ALL)
         doUsage("<on|off>", g_pszProgName, "setvideoacceleration");
+    if (eWhich  == VIDEO_FLAGS || eWhich == USAGE_ALL)
+        doUsage("<get|set|clear|delete> [hex mask]", g_pszProgName, "videoflags");
     if (LIST_CUST_MODES == eWhich || eWhich == USAGE_ALL)
         doUsage("", g_pszProgName, "listcustommodes");
     if (ADD_CUST_MODE == eWhich || eWhich == USAGE_ALL)
@@ -603,38 +606,120 @@ static RTEXITCODE handleSetVideoMode(int argc, char *argv[])
     return RTEXITCODE_SUCCESS;
 }
 
-HKEY getVideoKey(bool writable)
+static int checkVBoxVideoKey(HKEY hkeyVideo)
+{
+    char szValue[128];
+    DWORD len = sizeof(szValue);
+    DWORD dwKeyType;
+    LONG status = RegQueryValueExA(hkeyVideo, "Device Description", NULL, &dwKeyType,
+                                   (LPBYTE)szValue, &len);
+
+    if (status == ERROR_SUCCESS)
+    {
+        /* WDDM has additional chars after "Adapter" */
+        static char sszDeviceDescription[] = "VirtualBox Graphics Adapter";
+        if (_strnicmp(szValue, sszDeviceDescription, sizeof(sszDeviceDescription) - sizeof(char)) == 0)
+        {
+            return VINF_SUCCESS;
+        }
+    }
+
+    return VERR_NOT_FOUND;
+}
+
+static HKEY getVideoKey(bool writable)
 {
     HKEY hkeyDeviceMap = 0;
-    HKEY hkeyVideo = 0;
-    LONG status;
-
-    status = RegOpenKeyExA(HKEY_LOCAL_MACHINE, "HARDWARE\\DEVICEMAP\\VIDEO", 0, KEY_READ, &hkeyDeviceMap);
-    if ((status != ERROR_SUCCESS) || !hkeyDeviceMap)
+    LONG status = RegOpenKeyExA(HKEY_LOCAL_MACHINE, "HARDWARE\\DEVICEMAP\\VIDEO", 0, KEY_READ, &hkeyDeviceMap);
+    if (status != ERROR_SUCCESS || !hkeyDeviceMap)
     {
         VBoxControlError("Error opening video device map registry key!\n");
         return 0;
     }
-    char szVideoLocation[256];
+
+    HKEY hkeyVideo = 0;
+    ULONG iDevice;
     DWORD dwKeyType;
-    szVideoLocation[0] = 0;
-    DWORD len = sizeof(szVideoLocation);
-    status = RegQueryValueExA(hkeyDeviceMap, "\\Device\\Video0", NULL, &dwKeyType, (LPBYTE)szVideoLocation, &len);
+
     /*
-     * This value will start with a weird value: \REGISTRY\Machine
-     * Make sure this is true.
+     * Scan all '\Device\VideoX' REG_SZ keys to find VBox video driver entry.
+     * 'ObjectNumberList' REG_BINARY is an array of 32 bit device indexes (X).
      */
-    if (   (status == ERROR_SUCCESS)
-        && (dwKeyType == REG_SZ)
-        && (_strnicmp(szVideoLocation, "\\REGISTRY\\Machine", 17) == 0))
+
+    /* Get the 'ObjectNumberList' */
+    ULONG numDevices = 0;
+    DWORD adwObjectNumberList[256];
+    DWORD len = sizeof(adwObjectNumberList);
+    status = RegQueryValueExA(hkeyDeviceMap, "ObjectNumberList", NULL, &dwKeyType, (LPBYTE)&adwObjectNumberList[0], &len);
+
+    if (   status == ERROR_SUCCESS
+        && dwKeyType == REG_BINARY)
     {
-        /* open that branch */
-        status = RegOpenKeyExA(HKEY_LOCAL_MACHINE, &szVideoLocation[18], 0, KEY_READ | (writable ? KEY_WRITE : 0), &hkeyVideo);
+        numDevices = len / sizeof(DWORD);
     }
     else
     {
-        VBoxControlError("Error opening registry key '%s'\n", &szVideoLocation[18]);
+       /* The list might not exists. Use 'MaxObjectNumber' REG_DWORD and build a list. */
+       DWORD dwMaxObjectNumber = 0;
+       len = sizeof(dwMaxObjectNumber);
+       status = RegQueryValueExA(hkeyDeviceMap, "MaxObjectNumber", NULL, &dwKeyType, (LPBYTE)&dwMaxObjectNumber, &len);
+
+       if (   status == ERROR_SUCCESS
+           && dwKeyType == REG_DWORD)
+       {
+           /* 'MaxObjectNumber' is inclusive. */
+           numDevices = RT_MIN(dwMaxObjectNumber + 1, RT_ELEMENTS(adwObjectNumberList));
+           for (iDevice = 0; iDevice < numDevices; iDevice++)
+           {
+               adwObjectNumberList[iDevice] = iDevice;
+           }
+       }
     }
+    
+    if (numDevices == 0)
+    {
+        /* Always try '\Device\Video0' as the old code did. Enum can be used in this case in principle. */
+        adwObjectNumberList[0] = 0;
+        numDevices = 1;
+    }
+
+    /* Scan device entries */
+    for (iDevice = 0; iDevice < numDevices; iDevice++)
+    {
+        char szValueName[64];
+        RTStrPrintf(szValueName, sizeof(szValueName), "\\Device\\Video%u", adwObjectNumberList[iDevice]);
+
+        char szVideoLocation[256];
+        len = sizeof(szVideoLocation);
+        status = RegQueryValueExA(hkeyDeviceMap, szValueName, NULL, &dwKeyType, (LPBYTE)&szVideoLocation[0], &len);
+
+        /* This value starts with '\REGISTRY\Machine' */
+        if (   status == ERROR_SUCCESS
+            && dwKeyType == REG_SZ
+            && _strnicmp(szVideoLocation, "\\REGISTRY\\Machine", 17) == 0)
+        {
+            status = RegOpenKeyExA(HKEY_LOCAL_MACHINE, &szVideoLocation[18], 0,
+                                   KEY_READ | (writable ? KEY_WRITE : 0), &hkeyVideo);
+            if (status == ERROR_SUCCESS)
+            {
+                int rc = checkVBoxVideoKey(hkeyVideo);
+                if (RT_SUCCESS(rc))
+                {
+                    /* Found, return hkeyVideo to the caller. */
+                    break;
+                }
+
+                RegCloseKey(hkeyVideo);
+                hkeyVideo = 0;
+            }
+        }
+    }
+
+    if (hkeyVideo == 0)
+    {
+        VBoxControlError("Error opening video registry key!\n");
+    }
+
     RegCloseKey(hkeyDeviceMap);
     return hkeyVideo;
 }
@@ -692,6 +777,135 @@ static RTEXITCODE handleSetVideoAcceleration(int argc, char *argv[])
     return RTEXITCODE_SUCCESS;
 }
 
+static RTEXITCODE videoFlagsGet(void)
+{
+    HKEY hkeyVideo = getVideoKey(false);
+
+    if (hkeyVideo)
+    {
+        DWORD dwFlags = 0;
+        DWORD len = sizeof(dwFlags);
+        DWORD dwKeyType;
+        ULONG status = RegQueryValueExA(hkeyVideo, "VBoxVideoFlags", NULL, &dwKeyType, (LPBYTE)&dwFlags, &len);
+        if (status != ERROR_SUCCESS)
+            RTPrintf("Video flags: default\n");
+        else
+            RTPrintf("Video flags: 0x%08X\n", dwFlags);
+        RegCloseKey(hkeyVideo);
+        return RTEXITCODE_SUCCESS;
+    }
+
+    return RTEXITCODE_FAILURE;
+}
+
+static RTEXITCODE videoFlagsDelete(void)
+{
+    HKEY hkeyVideo = getVideoKey(true);
+
+    if (hkeyVideo)
+    {
+        ULONG status = RegDeleteValueA(hkeyVideo, "VBoxVideoFlags");
+        if (status != ERROR_SUCCESS)
+            VBoxControlError("Error %d deleting video flags.\n", status);
+        RegCloseKey(hkeyVideo);
+        return RTEXITCODE_SUCCESS;
+    }
+
+    return RTEXITCODE_FAILURE;
+}
+
+static RTEXITCODE videoFlagsModify(bool fSet, int argc, char *argv[])
+{
+    if (argc != 1)
+    {
+        VBoxControlError("Mask required.\n");
+        return RTEXITCODE_FAILURE;
+    }
+
+    uint32_t u32Mask = 0;
+    int rc = RTStrToUInt32Full(argv[0], 16, &u32Mask);
+    if (RT_FAILURE(rc))
+    {
+        VBoxControlError("Invalid video flags mask.\n");
+        return RTEXITCODE_FAILURE;
+    }
+
+    RTEXITCODE exitCode = RTEXITCODE_SUCCESS;
+
+    HKEY hkeyVideo = getVideoKey(true);
+    if (hkeyVideo)
+    {
+        DWORD dwFlags = 0;
+        DWORD len = sizeof(dwFlags);
+        DWORD dwKeyType;
+        ULONG status = RegQueryValueExA(hkeyVideo, "VBoxVideoFlags", NULL, &dwKeyType, (LPBYTE)&dwFlags, &len);
+        if (status != ERROR_SUCCESS)
+        {
+            dwFlags = 0;
+        }
+
+        dwFlags = fSet? (dwFlags | u32Mask):
+                        (dwFlags & ~u32Mask);
+
+        status = RegSetValueExA(hkeyVideo, "VBoxVideoFlags", 0, REG_DWORD, (LPBYTE)&dwFlags, sizeof(dwFlags));
+        if (status != ERROR_SUCCESS)
+        {
+            VBoxControlError("Error %d writing video flags.\n", status);
+            exitCode = RTEXITCODE_FAILURE;
+        }
+
+        RegCloseKey(hkeyVideo);
+    }
+    else
+    {
+        exitCode = RTEXITCODE_FAILURE;
+    }
+
+    return exitCode;
+}
+
+static RTEXITCODE handleVideoFlags(int argc, char *argv[])
+{
+    /* Must have a keyword and optional value (32 bit hex string). */
+    if (argc != 1 && argc != 2)
+    {
+        VBoxControlError("Invalid number of arguments.\n");
+        usage(VIDEO_FLAGS);
+        return RTEXITCODE_FAILURE;
+    }
+
+    RTEXITCODE exitCode = RTEXITCODE_SUCCESS;
+
+    if (RTStrICmp(argv[0], "get") == 0)
+    {
+        exitCode = videoFlagsGet();
+    }
+    else if (RTStrICmp(argv[0], "delete") == 0)
+    {
+        exitCode = videoFlagsDelete();
+    }
+    else if (RTStrICmp(argv[0], "set") == 0)
+    {
+        exitCode = videoFlagsModify(true, argc - 1, &argv[1]);
+    }
+    else if (RTStrICmp(argv[0], "clear") == 0)
+    {
+        exitCode = videoFlagsModify(false, argc - 1, &argv[1]);
+    }
+    else
+    {
+        VBoxControlError("Invalid command.\n");
+        exitCode = RTEXITCODE_FAILURE;
+    }
+
+    if (exitCode != RTEXITCODE_SUCCESS)
+    {
+        usage(VIDEO_FLAGS);
+    }
+
+    return exitCode;
+}
+
 #define MAX_CUSTOM_MODES 128
 
 /* the table of custom modes */
@@ -1533,6 +1747,7 @@ struct COMMANDHANDLER
 #if defined(RT_OS_WINDOWS) && !defined(VBOX_CONTROL_TEST)
     { "getvideoacceleration",   handleGetVideoAcceleration },
     { "setvideoacceleration",   handleSetVideoAcceleration },
+    { "videoflags",             handleVideoFlags },
     { "listcustommodes",        handleListCustomModes },
     { "addcustommode",          handleAddCustomMode },
     { "removecustommode",       handleRemoveCustomMode },
