@@ -80,32 +80,32 @@ typedef struct SYSFSCPUPATH
 const SYSFSCPUPATHCOMP g_aAcpiCpuPathLvl1[] =
 {
     /** LNXSYSTEM:<id> */
-    {true, "LNXSYSTM:"}
+    {true, "LNXSYSTM:*"}
 };
 
 /** Possible combinations of all path components for level 2. */
 const SYSFSCPUPATHCOMP g_aAcpiCpuPathLvl2[] =
 {
     /** device:<id> */
-    {true, "device:"},
+    {true, "device:*"},
     /** LNXSYBUS:<id> */
-    {true, "LNXSYBUS:"}
+    {true, "LNXSYBUS:*"}
 };
 
 /** Possible combinations of all path components for level 3 */
 const SYSFSCPUPATHCOMP g_aAcpiCpuPathLvl3[] =
 {
     /** ACPI0004:<id> */
-    {true, "ACPI0004:"}
+    {true, "ACPI0004:*"}
 };
 
 /** Possible combinations of all path components for level 4 */
 const SYSFSCPUPATHCOMP g_aAcpiCpuPathLvl4[] =
 {
     /** LNXCPU:<id> */
-    {true, "LNXCPU:"},
+    {true, "LNXCPU:*"},
     /** ACPI_CPU:<id> */
-    {true, "ACPI_CPU:"}
+    {true, "ACPI_CPU:*"}
 };
 
 /** All possible combinations. */
@@ -120,6 +120,18 @@ SYSFSCPUPATH g_aAcpiCpuPath[] =
     /** Level 4 */
     {ACPI_CPU_PATH_NOT_PROBED, g_aAcpiCpuPathLvl4, RT_ELEMENTS(g_aAcpiCpuPathLvl4), NULL, NULL},
 };
+
+/**
+ * Possible directories to get to the topology directory for reading core and package id.
+ *
+ * @remark: This is not part of the path above because the eject file is not in one of the directories
+ *          below and would make the hot unplug code fail.
+ */
+const char *g_apszTopologyPath[] =
+{
+    "sysdev",
+    "physical_node"
+};
 #endif
 
 #ifdef RT_OS_LINUX
@@ -172,6 +184,11 @@ static int VBoxServiceCpuHotPlugProbePath(void)
                 size_t cchName = strlen(pPathComponent->pcszName);
                 RTDIRENTRY DirFolderContent;
                 bool fFound = false;
+
+                /* Get rid of the * filter which is in the path component. */
+                if (pPathComponent->fNumberedSuffix)
+                    cchName--;
+
                 while (RT_SUCCESS(RTDirRead(pDirCurr, &DirFolderContent, NULL))) /* Assumption that szName has always enough space */
                 {
                     if (   DirFolderContent.cbName >= cchName
@@ -272,10 +289,25 @@ static int VBoxServiceCpuHotPlugGetACPIDevicePath(char **ppszPath, uint32_t idCp
                     if (iLvlCurr == RT_ELEMENTS(g_aAcpiCpuPath) - 1)
                     {
                         /* Get the sysdev */
-                        uint32_t idCore    = RTLinuxSysFsReadIntFile(10, "%s/sysdev/topology/core_id",
-                                                                     pszPathCurr);
-                        uint32_t idPackage = RTLinuxSysFsReadIntFile(10, "%s/sysdev/topology/physical_package_id",
-                                                                     pszPathCurr);
+                        uint32_t idCore = 0;
+                        uint32_t idPackage = 0;
+
+                        for (unsigned i = 0; i < RT_ELEMENTS(g_apszTopologyPath); i++)
+                        {
+                            int64_t i64Core    = RTLinuxSysFsReadIntFile(10, "%s/%s/topology/core_id",
+                                                                         pszPathCurr, g_apszTopologyPath[i]);
+                            int64_t i64Package = RTLinuxSysFsReadIntFile(10, "%s/%s/topology/physical_package_id",
+                                                                         pszPathCurr, g_apszTopologyPath[i]);
+
+                            if (   i64Core != -1
+                                && i64Package != -1)
+                            {
+                                idCore = (uint32_t)i64Core;
+                                idPackage = (uint32_t)i64Package;
+                                break;
+                            }
+                        }
+
                         if (   idCore    == idCpuCore
                             && idPackage == idCpuPackage)
                         {
