@@ -27,9 +27,14 @@
 #include <iprt/string.h>
 #include "compiler.h"
 #include "cursorstr.h"
+#include "servermd.h"
 
 #include "vboxvideo.h"
 
+#ifdef XORG_7X
+# include <stdlib.h>
+#endif
+
 #define VBOX_MAX_CURSOR_WIDTH 64
 #define VBOX_MAX_CURSOR_HEIGHT 64
 
