@@ -525,7 +525,19 @@ static int get_dns_addr_domain(PNATState pData, bool fVerbose,
             pDns->de_addr.s_addr = tmp_addr.s_addr;
             if ((pDns->de_addr.s_addr & RT_H2N_U32_C(IN_CLASSA_NET)) == RT_N2H_U32_C(INADDR_LOOPBACK & IN_CLASSA_NET))
             {
-                pDns->de_addr.s_addr = RT_H2N_U32(RT_N2H_U32(pData->special_addr.s_addr) | CTL_ALIAS);
+                if ((pDns->de_addr.s_addr) == RT_N2H_U32_C(INADDR_LOOPBACK))
+                    pDns->de_addr.s_addr = RT_H2N_U32(RT_N2H_U32(pData->special_addr.s_addr) | CTL_ALIAS);
+                else
+                {
+                    /* Modern Ubuntu register 127.0.1.1 as DNS server */
+                    LogRel(("NAT: DNS server %RTnaipv4 registration detected, switching to the host resolver.\n",
+                            pDns->de_addr.s_addr));
+                    RTMemFree(pDns);
+                    /* Releasing fetched DNS information. */
+                    slirp_release_dns_list(pData);
+                    pData->fUseHostResolver = 1;
+                    return VINF_SUCCESS;
+                }
             }
             TAILQ_INSERT_HEAD(&pData->pDnsList, pDns, de_list);
             cNameserversFound++;
