@@ -1,6 +1,6 @@
 /* $Id: DevHPET.cpp $ */
 /** @file
- * HPET virtual device - high precision event timer emulation
+ * HPET virtual device - High Precision Event Timer emulation
  */
 
 /*
@@ -15,6 +15,10 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+/* This implementation is based on the (generic) Intel IA-PC HPET specification
+ * and the Intel ICH9 datasheet.
+ */
+
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
@@ -41,10 +45,17 @@
  */
 
 /** Base address for MMIO. */
+/* On ICH9, it is 0xFED0x000 where 'x' is 0-3, default 0. We do not support
+ * relocation as the platform firmware is responsible for configuring the
+ * HPET base address and the OS isn't expected to move it.
+ * WARNING: This has to match the ACPI tables! */
 #define HPET_BASE                   0xfed00000
 
+/* HPET reserves a 1K range. */
+#define HPET_BAR_SIZE               0x1000
+
 /** The number of timers for PIIX4 / PIIX3. */
-#define HPET_NUM_TIMERS_PIIX        3
+#define HPET_NUM_TIMERS_PIIX        3   /* Minimal implementation. */
 /** The number of timers for ICH9. */
 #define HPET_NUM_TIMERS_ICH9        4
 
@@ -59,7 +70,7 @@
 #define HPET_CLK_PERIOD_ICH9        UINT32_C(69841279)
 
 /*
- * Femptosecods in nanosecond
+ * Femtosecods in a nanosecond
  */
 #define FS_PER_NS                   1000000
 
@@ -70,10 +81,8 @@
 #define HPET_TIMER_TYPE_EDGE        0
 
 /* Delivery mode */
-/* Via APIC */
-#define HPET_TIMER_DELIVERY_APIC    0
-/* Via FSB */
-#define HPET_TIMER_DELIVERY_FSB     1
+#define HPET_TIMER_DELIVERY_APIC    0   /* Delivery through APIC. */
+#define HPET_TIMER_DELIVERY_FSB     1   /* Delivery through FSB. */
 
 #define HPET_TIMER_CAP_FSB_INT_DEL (1 << 15)
 #define HPET_TIMER_CAP_PER_INT     (1 << 4)
@@ -81,32 +90,36 @@
 #define HPET_CFG_ENABLE          0x001 /* ENABLE_CNF */
 #define HPET_CFG_LEGACY          0x002 /* LEG_RT_CNF */
 
-#define HPET_ID                  0x000
-#define HPET_PERIOD              0x004
-#define HPET_CFG                 0x010
-#define HPET_STATUS              0x020
-#define HPET_COUNTER             0x0f0
-#define HPET_TN_CFG              0x000
-#define HPET_TN_CMP              0x008
-#define HPET_TN_ROUTE            0x010
+/* Register offsets in HPET space. */
+#define HPET_ID                  0x000  /* Device ID. */
+#define HPET_PERIOD              0x004  /* Clock period in femtoseconds. */
+#define HPET_CFG                 0x010  /* Configuration register. */
+#define HPET_STATUS              0x020  /* Status register. */
+#define HPET_COUNTER             0x0f0  /* Main HPET counter. */
+
+/* Timer N offsets (within each timer's space). */
+#define HPET_TN_CFG              0x000  /* Timer N configuration. */
+#define HPET_TN_CMP              0x008  /* Timer N comparator. */
+#define HPET_TN_ROUTE            0x010  /* Timer N interrupt route. */
+
 #define HPET_CFG_WRITE_MASK      0x3
 
-#define HPET_TN_INT_TYPE                      RT_BIT_64(1)
-#define HPET_TN_ENABLE                        RT_BIT_64(2)
-#define HPET_TN_PERIODIC                      RT_BIT_64(3)
-#define HPET_TN_PERIODIC_CAP                  RT_BIT_64(4)
-#define HPET_TN_SIZE_CAP                      RT_BIT_64(5)
-#define HPET_TN_SETVAL                        RT_BIT_64(6)
-#define HPET_TN_32BIT                         RT_BIT_64(8)
-#define HPET_TN_INT_ROUTE_MASK                UINT64_C(0x3e00)
-#define HPET_TN_CFG_WRITE_MASK                UINT64_C(0x3e46)
-#define HPET_TN_INT_ROUTE_SHIFT               9
-#define HPET_TN_INT_ROUTE_CAP_SHIFT           32
+#define HPET_TN_INT_TYPE                RT_BIT_64(1)
+#define HPET_TN_ENABLE                  RT_BIT_64(2)
+#define HPET_TN_PERIODIC                RT_BIT_64(3)
+#define HPET_TN_PERIODIC_CAP            RT_BIT_64(4)
+#define HPET_TN_SIZE_CAP                RT_BIT_64(5)
+#define HPET_TN_SETVAL                  RT_BIT_64(6)
+#define HPET_TN_32BIT                   RT_BIT_64(8)
+#define HPET_TN_INT_ROUTE_MASK          UINT64_C(0x3e00)
+#define HPET_TN_CFG_WRITE_MASK          UINT64_C(0x3e46)
+#define HPET_TN_INT_ROUTE_SHIFT         9
+#define HPET_TN_INT_ROUTE_CAP_SHIFT     32
+
 #define HPET_TN_CFG_BITS_READONLY_OR_RESERVED 0xffff80b1U
 
-/** Extract the timer count from the capabilities.
- * @todo Check if the mask is correct.  */
-#define HPET_CAP_GET_TIMERS(a_u32)          ( ((a_u32) >> 8) & 0xf )
+/** Extract the timer count from the capabilities. */
+#define HPET_CAP_GET_TIMERS(a_u32)      ( ((a_u32) >> 8) & 0x1f )
 
 /** The version of the saved state. */
 #define HPET_SAVED_STATE_VERSION       2
@@ -244,7 +257,7 @@ typedef struct HpetState
     /** Global device lock. */
     PDMCRITSECT          csLock;
 
-    /** If we emulate ICH9 HPET (different frequency & timer count). */
+    /** Whether we emulate ICH9 HPET (different frequency & timer count). */
     bool                 fIch9;
     uint8_t              padding0[7];
 } HpetState;
@@ -265,16 +278,6 @@ DECLINLINE(uint64_t) hpetInvalidValue(HpetTimer *pHpetTimer)
     return hpet32bitTimer(pHpetTimer) ? UINT32_MAX : UINT64_MAX;
 }
 
-DECLINLINE(uint32_t) hpetTimeAfter32(uint64_t a, uint64_t b)
-{
-    return ((int32_t)(b) - (int32_t)(a) <= 0);
-}
-
-DECLINLINE(uint32_t) hpetTimeAfter64(uint64_t a, uint64_t b)
-{
-    return ((int64_t)(b) - (int64_t)(a) <= 0);
-}
-
 DECLINLINE(uint64_t) hpetTicksToNs(HpetState *pThis, uint64_t value)
 {
     return ASMMultU64ByU32DivByU32(value,  pThis->u32Period, FS_PER_NS);
@@ -346,22 +349,14 @@ DECLINLINE(uint64_t) hpetComputeDiff(HpetTimer *pHpetTimer,
 
 static void hpetAdjustComparator(HpetTimer *pHpetTimer, uint64_t u64Now)
 {
-  uint64_t u64Period = pHpetTimer->u64Period;
-  if (   (pHpetTimer->u64Config & HPET_TN_PERIODIC)
-      && u64Period != 0)
-  {
-      /* While loop is suboptimal */
-      if (hpet32bitTimer(pHpetTimer))
-      {
-          while (hpetTimeAfter32(u64Now, pHpetTimer->u64Cmp))
-              pHpetTimer->u64Cmp = (uint32_t)(pHpetTimer->u64Cmp + u64Period);
-      }
-      else
-      {
-          while (hpetTimeAfter64(u64Now, pHpetTimer->u64Cmp))
-              pHpetTimer->u64Cmp += u64Period;
-      }
-  }
+    uint64_t    u64Period = pHpetTimer->u64Period;
+
+    if ((pHpetTimer->u64Config & HPET_TN_PERIODIC) && u64Period)
+    {
+          uint64_t  cPeriods = (u64Now - pHpetTimer->u64Cmp) / u64Period;
+
+          pHpetTimer->u64Cmp += (cPeriods + 1) * u64Period;
+    }
 }
 
 
@@ -436,14 +431,15 @@ static void hpetProgramTimer(HpetTimer *pHpetTimer)
  * @param   iTimerReg           The index of the timer register to read.
  * @param   pu32Value           Where to return the register value.
  *
- * @remarks ASSUMES the caller does holds the HPET lock.
+ * @remarks ASSUMES the caller holds the HPET lock.
  */
 static int hpetTimerRegRead32(HpetState const *pThis, uint32_t iTimerNo, uint32_t iTimerReg, uint32_t *pu32Value)
 {
     Assert(PDMCritSectIsOwner(&pThis->csLock));
 
-    if (iTimerNo >= HPET_CAP_GET_TIMERS(pThis->u32Capabilities))
-    {
+    if (   iTimerNo >= HPET_CAP_GET_TIMERS(pThis->u32Capabilities)  /* The second check is only to satisfy Parfait; */
+        || iTimerNo >= RT_ELEMENTS(pThis->aTimers) )                /* in practice, the number of configured timers */
+    {                                                               /* will always be <= aTimers elements. */
         static unsigned s_cOccurences = 0;
         if (s_cOccurences++ < 10)
             LogRel(("HPET: using timer above configured range: %d\n", iTimerNo));
@@ -510,7 +506,8 @@ static int hpetTimerRegWrite32(HpetState *pThis, uint32_t iTimerNo, uint32_t iTi
 {
     Assert(!PDMCritSectIsOwner(&pThis->csLock) || TMTimerIsLockOwner(pThis->aTimers[0].CTX_SUFF(pTimer)));
 
-    if (iTimerNo >= HPET_CAP_GET_TIMERS(pThis->u32Capabilities))
+    if (   iTimerNo >= HPET_CAP_GET_TIMERS(pThis->u32Capabilities)
+        || iTimerNo >= RT_ELEMENTS(pThis->aTimers) )    /* Parfait - see above. */
     {
         static unsigned s_cOccurences = 0;
         if (s_cOccurences++ < 10)
@@ -524,10 +521,9 @@ static int hpetTimerRegWrite32(HpetState *pThis, uint32_t iTimerNo, uint32_t iTi
         case HPET_TN_CFG:
         {
             DEVHPET_LOCK_RETURN(pThis, VINF_IOM_HC_MMIO_WRITE);
-            Log(("write HPET_TN_CFG: %d: %x\n", iTimerNo, u32NewValue));
-            uint64_t const iOldValue = (uint32_t)pHpetTimer->u64Config;
+            uint64_t    u64Mask = HPET_TN_CFG_WRITE_MASK;
 
-            uint64_t u64Mask = HPET_TN_CFG_WRITE_MASK;
+            Log(("write HPET_TN_CFG: %d: %x\n", iTimerNo, u32NewValue));
             if (pHpetTimer->u64Config & HPET_TN_PERIODIC_CAP)
                 u64Mask |= HPET_TN_PERIODIC;
 
@@ -551,12 +547,12 @@ static int hpetTimerRegWrite32(HpetState *pThis, uint32_t iTimerNo, uint32_t iTi
             }
 
             /* We only care about lower 32-bits so far */
-            pHpetTimer->u64Config = hpetUpdateMasked(u32NewValue, iOldValue, u64Mask);
+            pHpetTimer->u64Config = hpetUpdateMasked(u32NewValue, pHpetTimer->u64Config, u64Mask);
             DEVHPET_UNLOCK(pThis);
             break;
         }
 
-        case HPET_TN_CFG + 4: /* Interrupt capabilities */
+        case HPET_TN_CFG + 4: /* Interrupt capabilities - read only. */
         {
             Log(("write HPET_TN_CFG + 4, useless\n"));
             break;
@@ -568,11 +564,8 @@ static int hpetTimerRegWrite32(HpetState *pThis, uint32_t iTimerNo, uint32_t iTi
             Log(("write HPET_TN_CMP on %d: %#x\n", iTimerNo, u32NewValue));
 
             if (pHpetTimer->u64Config & HPET_TN_PERIODIC)
-            {
-                u32NewValue &= hpetInvalidValue(pHpetTimer) >> 1; /** @todo check this in the docs and add a not why? */
-                pHpetTimer->u64Period = RT_MAKE_U64(u32NewValue, pHpetTimer->u64Period);
-            }
-            pHpetTimer->u64Cmp     = RT_MAKE_U64(u32NewValue, pHpetTimer->u64Cmp);
+                pHpetTimer->u64Period = RT_MAKE_U64(u32NewValue, RT_HI_U32(pHpetTimer->u64Period));
+            pHpetTimer->u64Cmp     = RT_MAKE_U64(u32NewValue, RT_HI_U32(pHpetTimer->u64Cmp));
             pHpetTimer->u64Config &= ~HPET_TN_SETVAL;
             Log2(("after HPET_TN_CMP cmp=%#llx per=%#llx\n", pHpetTimer->u64Cmp, pHpetTimer->u64Period));
 
@@ -589,8 +582,8 @@ static int hpetTimerRegWrite32(HpetState *pThis, uint32_t iTimerNo, uint32_t iTi
             if (!hpet32bitTimer(pHpetTimer))
             {
                 if (pHpetTimer->u64Config & HPET_TN_PERIODIC)
-                    pHpetTimer->u64Period = RT_MAKE_U64(pHpetTimer->u64Period, u32NewValue);
-                pHpetTimer->u64Cmp = RT_MAKE_U64(pHpetTimer->u64Cmp, u32NewValue);
+                    pHpetTimer->u64Period = RT_MAKE_U64(RT_LO_U32(pHpetTimer->u64Period), u32NewValue);
+                pHpetTimer->u64Cmp = RT_MAKE_U64(RT_LO_U32(pHpetTimer->u64Cmp), u32NewValue);
 
                 Log2(("after HPET_TN_CMP+4 cmp=%llx per=%llx tmr=%d\n", pHpetTimer->u64Cmp, pHpetTimer->u64Period, iTimerNo));
 
@@ -826,7 +819,7 @@ static int hpetConfigRegWrite32(HpetState *pThis, uint32_t idxReg, uint32_t u32N
         case HPET_COUNTER:
         {
             DEVHPET_LOCK_RETURN(pThis, VINF_IOM_HC_MMIO_WRITE);
-            pThis->u64HpetCounter = RT_MAKE_U64(u32NewValue, pThis->u64HpetCounter);
+            pThis->u64HpetCounter = RT_MAKE_U64(u32NewValue, RT_HI_U32(pThis->u64HpetCounter));
             Log(("write HPET_COUNTER: %#x -> %llx\n", u32NewValue, pThis->u64HpetCounter));
             DEVHPET_UNLOCK(pThis);
             break;
@@ -835,7 +828,7 @@ static int hpetConfigRegWrite32(HpetState *pThis, uint32_t idxReg, uint32_t u32N
         case HPET_COUNTER + 4:
         {
             DEVHPET_LOCK_RETURN(pThis, VINF_IOM_HC_MMIO_WRITE);
-            pThis->u64HpetCounter = RT_MAKE_U64(pThis->u64HpetCounter, u32NewValue);
+            pThis->u64HpetCounter = RT_MAKE_U64(RT_LO_U32(pThis->u64HpetCounter), u32NewValue);
             Log(("write HPET_COUNTER + 4: %#x -> %llx\n", u32NewValue, pThis->u64HpetCounter));
             DEVHPET_UNLOCK(pThis);
             break;
@@ -1028,7 +1021,7 @@ PDMBOTHCBDECL(int) hpetMMIOWrite(PPDMDEVINS pDevIns, void *pvUser, RTGCPHYS GCPh
 static uint32_t hpetTimerCbGetIrq(struct HpetTimer const *pHpetTimer)
 {
     /*
-     * Per spec, in legacy mode HPET timers wired as:
+     * Per spec, in legacy mode the HPET timers are wired as follows:
      *   timer 0: IRQ0 for PIC and IRQ2 for APIC
      *   timer 1: IRQ8 for both PIC and APIC
      *
@@ -1087,18 +1080,20 @@ static DECLCALLBACK(void) hpetTimerCb(PPDMDEVINS pDevIns, PTMTIMER pTimer, void
     uint64_t   u64CurTick = hpetGetTicks(pThis);
     uint64_t   u64Diff;
 
-    if ((pHpetTimer->u64Config & HPET_TN_PERIODIC) && (u64Period != 0))
+    if (pHpetTimer->u64Config & HPET_TN_PERIODIC)
     {
-        hpetAdjustComparator(pHpetTimer, u64CurTick);
-
-        u64Diff = hpetComputeDiff(pHpetTimer, u64CurTick);
-
-        Log4(("HPET: periodical: next in %llu\n", hpetTicksToNs(pThis, u64Diff)));
-        TMTimerSetNano(pTimer, hpetTicksToNs(pThis, u64Diff));
+        if (u64Period) {
+            hpetAdjustComparator(pHpetTimer, u64CurTick);
+    
+            u64Diff = hpetComputeDiff(pHpetTimer, u64CurTick);
+    
+            Log4(("HPET: periodic: next in %llu\n", hpetTicksToNs(pThis, u64Diff)));
+            TMTimerSetNano(pTimer, hpetTicksToNs(pThis, u64Diff));
+        }
     }
-    else if (    hpet32bitTimer(pHpetTimer)
-             && !(pHpetTimer->u64Config & HPET_TN_PERIODIC))
+    else if (hpet32bitTimer(pHpetTimer))
     {
+        /* For 32-bit non-periodic timers, generate wrap-around interrupts. */
         if (pHpetTimer->u8Wrap)
         {
             u64Diff = hpetComputeDiff(pHpetTimer, u64CurTick);
@@ -1437,14 +1432,14 @@ static DECLCALLBACK(int) hpetConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMN
      * Register the MMIO range, PDM API requests page aligned
      * addresses and sizes.
      */
-    rc = PDMDevHlpMMIORegister(pDevIns, HPET_BASE, 0x1000, pThis,
+    rc = PDMDevHlpMMIORegister(pDevIns, HPET_BASE, HPET_BAR_SIZE, pThis,
                                IOMMMIO_FLAGS_READ_PASSTHRU | IOMMMIO_FLAGS_WRITE_PASSTHRU,
                                hpetMMIOWrite, hpetMMIORead, "HPET Memory");
     AssertRCReturn(rc, rc);
 
     if (fRCEnabled)
     {
-        rc = PDMDevHlpMMIORegisterRC(pDevIns, HPET_BASE, 0x1000, NIL_RTRCPTR /*pvUser*/, "hpetMMIOWrite", "hpetMMIORead");
+        rc = PDMDevHlpMMIORegisterRC(pDevIns, HPET_BASE, HPET_BAR_SIZE, NIL_RTRCPTR /*pvUser*/, "hpetMMIOWrite", "hpetMMIORead");
         AssertRCReturn(rc, rc);
 
         pThis->pHpetHlpRC = pThis->pHpetHlpR3->pfnGetRCHelpers(pDevIns);
@@ -1453,7 +1448,7 @@ static DECLCALLBACK(int) hpetConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMN
 
     if (fR0Enabled)
     {
-        rc = PDMDevHlpMMIORegisterR0(pDevIns, HPET_BASE, 0x1000, NIL_RTR0PTR /*pvUser*/,
+        rc = PDMDevHlpMMIORegisterR0(pDevIns, HPET_BASE, HPET_BAR_SIZE, NIL_RTR0PTR /*pvUser*/,
                                      "hpetMMIOWrite", "hpetMMIORead");
         AssertRCReturn(rc, rc);
 
