@@ -275,7 +275,7 @@ static int disCoreOne(PDISCPUSTATE pCpu, RTUINTPTR InstructionAddr, unsigned *pc
         pCpu->opmode     = pCpu->mode;
     }
 
-#ifdef IN_RING3
+#if defined(IN_RING3) && !defined(IN_SUP_HARDENED_R3)
 # ifndef __L4ENV__  /* Unfortunately, we have no exception handling in l4env */
     try
 # else
@@ -384,7 +384,7 @@ static int disCoreOne(PDISCPUSTATE pCpu, RTUINTPTR InstructionAddr, unsigned *pc
             break;
         }
     }
-#ifdef IN_RING3
+#if defined(IN_RING3) && !defined(IN_SUP_HARDENED_R3)
 # ifndef __L4ENV__
     catch(...)
 # else
@@ -2420,7 +2420,7 @@ void disasmModRMReg(PDISCPUSTATE pCpu, PCOPCODE pOp, unsigned idx, POP_PARAMETER
         break;
 
     default:
-#ifdef IN_RING3
+#if defined(IN_RING3) && !defined(IN_SUP_HARDENED_R3)
         Log(("disasmModRMReg %x:%x failed!!\n", type, subtype));
         DIS_THROW(ExceptionInvalidModRM);
 #else
@@ -2449,7 +2449,7 @@ void disasmModRMSReg(PDISCPUSTATE pCpu, PCOPCODE pOp, unsigned idx, POP_PARAMETE
 #if 0 //def DEBUG_Sander
     AssertMsg(idx < RT_ELEMENTS(szModRMSegReg), ("idx=%d\n", idx));
 #endif
-#ifdef IN_RING3
+#if defined(IN_RING3) && !defined(IN_SUP_HARDENED_R3)
     if (idx >= RT_ELEMENTS(szModRMSegReg))
     {
         Log(("disasmModRMSReg %d failed!!\n", idx));
