@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -26,7 +26,7 @@
 # include <VBox/com/array.h>
 # include <VBox/com/ErrorInfo.h>
 # include <VBox/com/errorprint.h>
-# include <VBox/com/EventQueue.h>
+# include <VBox/com/NativeEventQueue.h>
 
 # include <VBox/com/VirtualBox.h>
 #endif /* !VBOX_ONLY_DOCS */
@@ -89,7 +89,7 @@ HRESULT showProgress(ComPtr<IProgress> progress)
     ULONG ulLastOperation = (ULONG)-1;
     Bstr bstrOperationDescription;
 
-    EventQueue::getMainEventQueue()->processEventQueue(0);
+    NativeEventQueue::getMainEventQueue()->processEventQueue(0);
 
     ULONG cOperations = 1;
     HRESULT hrc = progress->COMGETTER(OperationCount)(&cOperations);
@@ -190,7 +190,7 @@ HRESULT showProgress(ComPtr<IProgress> progress)
         /* make sure the loop is not too tight */
         progress->WaitForCompletion(100);
 
-        EventQueue::getMainEventQueue()->processEventQueue(0);
+        NativeEventQueue::getMainEventQueue()->processEventQueue(0);
         hrc = progress->COMGETTER(Completed(&fCompleted));
     }
 
@@ -501,7 +501,7 @@ int main(int argc, char *argv[])
          * state file (if the machine was in the Saved state before). */
         session->UnlockMachine();
 
-        EventQueue::getMainEventQueue()->processEventQueue(0);
+        NativeEventQueue::getMainEventQueue()->processEventQueue(0);
 
     // end "all-stuff" scope
     ///////////////////////////////////////////////////////////////////////////
