@@ -150,7 +150,7 @@ DECLASM(int) VBoxDrvOpen(uint16_t sfn)
     /*
      * Create a new session.
      */
-    rc = supdrvCreateSession(&g_DevExt, true /* fUser */, &pSession);
+    rc = supdrvCreateSession(&g_DevExt, true /* fUser */, true /* fUnrestricted */, &pSession);
     if (RT_SUCCESS(rc))
     {
         pSession->sfn = sfn;
@@ -223,7 +223,7 @@ DECLASM(int) VBoxDrvClose(uint16_t sfn)
     /*
      * Close the session.
      */
-    supdrvCloseSession(&g_DevExt, pSession);
+    supdrvSessionRelease(pSession);
     return 0;
 }
 
@@ -246,6 +246,9 @@ DECLASM(int) VBoxDrvIOCtlFast(uint16_t sfn, uint8_t iFunction)
         while (     pSession
                &&   (   pSession->sfn != sfn
                      || pSession->Process != Process));
+
+        if (RT_LIKELY(pSession))
+            supdrvSessionRetain(pSession);
     }
     RTSpinlockReleaseNoInts(g_Spinlock, &Tmp);
     if (RT_UNLIKELY(!pSession))
@@ -258,6 +261,7 @@ DECLASM(int) VBoxDrvIOCtlFast(uint16_t sfn, uint8_t iFunction)
      * Dispatch the fast IOCtl.
      */
     supdrvIOCtlFast(iFunction, 0, &g_DevExt, pSession);
+    supdrvSessionRelease(pSession);
     return 0;
 }
 
@@ -280,6 +284,9 @@ DECLASM(int) VBoxDrvIOCtl(uint16_t sfn, uint8_t iCat, uint8_t iFunction, void *p
         while (     pSession
                &&   (   pSession->sfn != sfn
                      || pSession->Process != Process));
+
+        if (RT_LIKELY(pSession))
+            supdrvSessionRetain(pSession);
     }
     RTSpinlockReleaseNoInts(g_Spinlock, &Tmp);
     if (!pSession)
@@ -331,7 +338,7 @@ DECLASM(int) VBoxDrvIOCtl(uint16_t sfn, uint8_t iCat, uint8_t iFunction, void *p
                 /*
                  * Process the IOCtl.
                  */
-                rc = supdrvIOCtl(iFunction, &g_DevExt, pSession, pHdr);
+                rc = supdrvIOCtl(iFunction, &g_DevExt, pSession, pHdr, cbReq);
             }
             else
             {
@@ -349,12 +356,34 @@ DECLASM(int) VBoxDrvIOCtl(uint16_t sfn, uint8_t iCat, uint8_t iFunction, void *p
          * Unlock and return.
          */
         int rc2 = KernVMUnlock(&Lock);
-        AssertMsg(!rc2, ("rc2=%d\n", rc2)); NOREF(rc2);
-
-        Log2(("VBoxDrvIOCtl: returns %d\n", rc));
-        return rc;
+        AssertMsg(!rc2, ("rc2=%d\n", rc2)); NOREF(rc2);s
     }
-    return VERR_NOT_SUPPORTED;
+    else
+        rc = VERR_NOT_SUPPORTED;
+
+    supdrvSessionRelease(pSession);
+
+    Log2(("VBoxDrvIOCtl: returns %d\n", rc));
+    return rc;
+}
+
+
+void VBOXCALL supdrvOSCleanupSession(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession)
+{
+    NOREF(pDevExt);
+    NOREF(pSession);
+}
+
+
+void VBOXCALL supdrvOSSessionHashTabInserted(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, void *pvUser)
+{
+    NOREF(pDevExt); NOREF(pSession); NOREF(pvUser);
+}
+
+
+void VBOXCALL supdrvOSSessionHashTabRemoved(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, void *pvUser)
+{
+    NOREF(pDevExt); NOREF(pSession); NOREF(pvUser);
 }
 
 
