@@ -53,6 +53,7 @@
 #include <iprt/alloca.h>
 #include <iprt/time.h>
 #include <iprt/net.h>
+#include <iprt/list.h>
 
 RT_C_DECLS_BEGIN
 /* ntddk.h has a missing #pragma pack(), work around it
@@ -372,6 +373,12 @@ typedef struct VBOXNETFLTGLOBALS_WIN
 #ifndef VBOXNETADP
     /* Protocol info */
     VBOXNETFLTGLOBALS_PT Pt;
+    /** lock protecting the filter list */
+    NDIS_SPIN_LOCK lockFilters;
+    /** the head of filter list */
+    RTLISTANCHOR listFilters;
+    /** IP address change notifier handle */
+    HANDLE hNotifier;
 #endif
 } VBOXNETFLTGLOBALS_WIN, *PVBOXNETFLTGLOBALS_WIN;
 
@@ -454,6 +461,8 @@ typedef struct VBOXNETFLTWIN
     ULONG fSetFilterBuffer;
     /** packet filter flags set by us */
     ULONG fOurSetFilter;
+    /** our own list of filters, needed by notifier */
+    RTLISTNODE node;
 #else
     volatile ULONG cTxSuccess;
     volatile ULONG cRxSuccess;
