@@ -102,197 +102,423 @@ void vbsfStripLastComponent(char *pszFullPath, uint32_t cbFullPathRoot)
     LogFlowFunc(("%s, %s, %s\n", pszFullPath, delimLast, delimSecondLast));
 }
 
+/**
+ * Corrects the casing of the final component
+ *
+ * @returns
+ * @param   pClient             .
+ * @param   pszFullPath         .
+ * @param   pszStartComponent   .
+ */
 static int vbsfCorrectCasing(SHFLCLIENTDATA *pClient, char *pszFullPath, char *pszStartComponent)
 {
-    PRTDIRENTRYEX  pDirEntry = NULL;
-    uint32_t       cbDirEntry, cbComponent;
-    int            rc = VERR_FILE_NOT_FOUND;
-    PRTDIR         hSearch = 0;
-    char           szWildCard[4];
-
     Log2(("vbsfCorrectCasing: %s %s\n", pszFullPath, pszStartComponent));
 
-    cbComponent = (uint32_t) strlen(pszStartComponent);
+    AssertReturn((uintptr_t)pszFullPath < (uintptr_t)pszStartComponent - 1U, VERR_INTERNAL_ERROR_2);
+    AssertReturn(pszStartComponent[-1] == RTPATH_DELIMITER, VERR_INTERNAL_ERROR_5);
 
-    cbDirEntry = 4096;
-    pDirEntry  = (PRTDIRENTRYEX)RTMemAlloc(cbDirEntry);
-    if (pDirEntry == 0)
-    {
-        AssertFailed();
+    /*
+     * Allocate a buffer that can hold really long file name entries as well as
+     * the initial search pattern.
+     */
+    size_t cchComponent = strlen(pszStartComponent);
+    size_t cchParentDir = pszStartComponent - pszFullPath;
+    size_t cchFullPath  = cchParentDir + cchComponent;
+    Assert(strlen(pszFullPath) == cchFullPath);
+
+    size_t cbDirEntry   = 4096;
+    if (cchFullPath + 4 > cbDirEntry - RT_OFFSETOF(RTDIRENTRYEX, szName))
+        cbDirEntry = RT_OFFSETOF(RTDIRENTRYEX, szName) + cchFullPath + 4;
+
+    PRTDIRENTRYEX pDirEntry = (PRTDIRENTRYEX)RTMemAlloc(cbDirEntry);
+    if (pDirEntry == NULL)
         return VERR_NO_MEMORY;
-    }
 
-    /** @todo this is quite inefficient, especially for directories with many files */
-    Assert(pszFullPath < pszStartComponent-1);
-    Assert(*(pszStartComponent-1) == RTPATH_DELIMITER);
-    *(pszStartComponent-1) = 0;
-    strcpy(pDirEntry->szName, pszFullPath);
-    szWildCard[0] = RTPATH_DELIMITER;
-    szWildCard[1] = '*';
-    szWildCard[2] = 0;
-    strcat(pDirEntry->szName, szWildCard);
-
-    rc = RTDirOpenFiltered(&hSearch, pDirEntry->szName, RTDIRFILTER_WINNT);
-    *(pszStartComponent-1) = RTPATH_DELIMITER;
-    if (RT_FAILURE(rc))
-        goto end;
-
-    for (;;)
+    /*
+     * Construct the search criteria in the szName member of pDirEntry.
+     */
+    /** @todo This is quite inefficient, especially for directories with many
+     *        files.  If any of the typically case sensitive host systems start
+     *        supporting opendir wildcard filters, it would make sense to build
+     *        one here with '?' for case foldable charaters. */
+    /** @todo Use RTDirOpen here and drop the whole uncessary path copying? */
+    int rc = RTPathJoinEx(pDirEntry->szName, cbDirEntry - RT_OFFSETOF(RTDIRENTRYEX, szName),
+                          pszFullPath, cchParentDir,
+                          RT_STR_TUPLE("*"));
+    AssertRC(rc);
+    if (RT_SUCCESS(rc))
     {
-        size_t cbDirEntrySize = cbDirEntry;
-
-        rc = RTDirReadEx(hSearch, pDirEntry, &cbDirEntrySize, RTFSOBJATTRADD_NOTHING, SHFL_RT_LINK(pClient));
-        if (rc == VERR_NO_MORE_FILES)
-            break;
-
-        if (   rc != VINF_SUCCESS
-            && rc != VWRN_NO_DIRENT_INFO)
+        PRTDIR hSearch = NULL;
+        rc = RTDirOpenFiltered(&hSearch, pDirEntry->szName, RTDIRFILTER_WINNT);
+        if (RT_SUCCESS(rc))
         {
-            AssertFailed();
-            if (   rc == VERR_NO_TRANSLATION
-                || rc == VERR_INVALID_UTF8_ENCODING)
-                continue;
-            break;
-        }
+            for (;;)
+            {
+                size_t cbDirEntrySize = cbDirEntry;
 
-        Log2(("vbsfCorrectCasing: found %s\n", &pDirEntry->szName[0]));
-        if (    pDirEntry->cbName == cbComponent
-            &&  !RTStrICmp(pszStartComponent, &pDirEntry->szName[0]))
-        {
-            Log(("Found original name %s (%s)\n", &pDirEntry->szName[0], pszStartComponent));
-            strcpy(pszStartComponent, &pDirEntry->szName[0]);
-            rc = VINF_SUCCESS;
-            break;
+                rc = RTDirReadEx(hSearch, pDirEntry, &cbDirEntrySize, RTFSOBJATTRADD_NOTHING, SHFL_RT_LINK(pClient));
+                if (rc == VERR_NO_MORE_FILES)
+                    break;
+
+                if (   rc != VINF_SUCCESS
+                    && rc != VWRN_NO_DIRENT_INFO)
+                {
+                    AssertFailed();
+                    if (   rc == VERR_NO_TRANSLATION
+                        || rc == VERR_INVALID_UTF8_ENCODING)
+                        continue;
+                    break;
+                }
+
+                Log2(("vbsfCorrectCasing: found %s\n", &pDirEntry->szName[0]));
+                if (    pDirEntry->cbName == cchComponent
+                    &&  !RTStrICmp(pszStartComponent, &pDirEntry->szName[0]))
+                {
+                    Log(("Found original name %s (%s)\n", &pDirEntry->szName[0], pszStartComponent));
+                    strcpy(pszStartComponent, &pDirEntry->szName[0]);
+                    rc = VINF_SUCCESS;
+                    break;
+                }
+            }
+
+            RTDirClose(hSearch);
         }
     }
 
-end:
     if (RT_FAILURE(rc))
-        Log(("vbsfCorrectCasing %s failed with %d\n", pszStartComponent, rc));
+        Log(("vbsfCorrectCasing %s failed with %Rrc\n", pszStartComponent, rc));
 
-    if (pDirEntry)
-        RTMemFree(pDirEntry);
+    RTMemFree(pDirEntry);
 
-    if (hSearch)
-        RTDirClose(hSearch);
     return rc;
 }
 
-static int vbsfPathCheck(const char *pUtf8Path, size_t cbPath)
+/* Temporary stand-in for RTPathExistEx. */
+static int vbsfQueryExistsEx(const char *pszPath, uint32_t fFlags)
 {
-    int rc = VINF_SUCCESS;
+#if 0 /** @todo Fix the symlink issue on windows! */
+    return RTPathExistsEx(pszPath, fFlags);
+#else
+    RTFSOBJINFO IgnInfo;
+    return RTPathQueryInfoEx(pszPath, &IgnInfo, RTFSOBJATTRADD_NOTHING, fFlags);
+#endif
+}
 
-    /* The pUtf8Path is what the guest sent. Verify that the path is within the root.
-     * Count '..' and other path components and check that we do not go over the root.
+/**
+ * Helper for vbsfBuildFullPath that performs case corrections on the path
+ * that's being build.
+ *
+ * @returns VINF_SUCCESS at the moment.
+ * @param   pClient                 The client data.
+ * @param   pszFullPath             Pointer to the full path.  This is the path
+ *                                  which may need case corrections.  The
+ *                                  corrections will be applied in place.
+ * @param   cchFullPath             The length of the full path.
+ * @param   fWildCard               Whether the last component may contain
+ *                                  wildcards and thus might require exclusion
+ *                                  from the case correction.
+ * @param   fPreserveLastComponent  Always exclude the last component from case
+ *                                  correction if set.
+ */
+static int vbsfCorrectPathCasing(SHFLCLIENTDATA *pClient, char *pszFullPath, size_t cchFullPath,
+                                 bool fWildCard, bool fPreserveLastComponent)
+{
+    /*
+     * Hide the last path component if it needs preserving.  This is required
+     * in the following cases:
+     *    - Contains the wildcard(s).
+     *    - Is a 'rename' target.
+     */
+    char *pszLastComponent = NULL;
+    if (fWildCard || fPreserveLastComponent)
+    {
+        char *pszSrc = pszFullPath + cchFullPath - 1;
+        Assert(strchr(pszFullPath, '\0') == pszSrc + 1);
+        while ((uintptr_t)pszSrc > (uintptr_t)pszFullPath)
+        {
+            if (*pszSrc == RTPATH_DELIMITER)
+                break;
+            pszSrc--;
+        }
+        if (*pszSrc == RTPATH_DELIMITER)
+        {
+            if (   fPreserveLastComponent
+                /* Or does it really have wildcards? */
+                || strchr(pszSrc + 1, '*') != NULL
+                || strchr(pszSrc + 1, '?') != NULL
+                || strchr(pszSrc + 1, '>') != NULL
+                || strchr(pszSrc + 1, '<') != NULL
+                || strchr(pszSrc + 1, '"') != NULL )
+            {
+                pszLastComponent = pszSrc;
+                *pszLastComponent = '\0';
+            }
+        }
+    }
+
+    /*
+     * If the path/file doesn't exist, we need to attempt case correcting it.
+     */
+    /** @todo Don't check when creating files or directories; waste of time. */
+    int rc = vbsfQueryExistsEx(pszFullPath, SHFL_RT_LINK(pClient));
+    if (rc == VERR_FILE_NOT_FOUND || rc == VERR_PATH_NOT_FOUND)
+    {
+        Log(("Handle case insensitive guest fs on top of host case sensitive fs for %s\n", pszFullPath));
+
+        /*
+         * Work from the end of the path to find a partial path that's valid.
+         */
+        char *pszSrc = pszLastComponent ? pszLastComponent - 1 : pszFullPath + cchFullPath - 1;
+        Assert(strchr(pszFullPath, '\0') == pszSrc + 1);
+
+        while ((uintptr_t)pszSrc > (uintptr_t)pszFullPath)
+        {
+            if (*pszSrc == RTPATH_DELIMITER)
+            {
+                *pszSrc = '\0';
+                rc = vbsfQueryExistsEx(pszFullPath, SHFL_RT_LINK(pClient));
+                *pszSrc = RTPATH_DELIMITER;
+                if (RT_SUCCESS(rc))
+                {
+#ifdef DEBUG
+                    *pszSrc = '\0';
+                    Log(("Found valid partial path %s\n", pszFullPath));
+                    *pszSrc = RTPATH_DELIMITER;
+#endif
+                    break;
+                }
+            }
+
+            pszSrc--;
+        }
+        Assert(*pszSrc == RTPATH_DELIMITER && RT_SUCCESS(rc));
+        if (   *pszSrc == RTPATH_DELIMITER
+            && RT_SUCCESS(rc))
+        {
+            /*
+             * Turn around and work the other way case correcting the components.
+             */
+            pszSrc++;
+            for (;;)
+            {
+                bool fEndOfString = true;
+
+                /* Find the end of the component. */
+                char *pszEnd = pszSrc;
+                while (*pszEnd)
+                {
+                    if (*pszEnd == RTPATH_DELIMITER)
+                        break;
+                    pszEnd++;
+                }
+
+                if (*pszEnd == RTPATH_DELIMITER)
+                {
+                    fEndOfString = false;
+                    *pszEnd = '\0';
+#if 0 /** @todo Please, double check this. The original code is in the #if 0, what I hold as correct is in the #else. */
+                    rc = RTPathQueryInfoEx(pszSrc, &info, RTFSOBJATTRADD_NOTHING, SHFL_RT_LINK(pClient));
+#else
+                    rc = vbsfQueryExistsEx(pszFullPath, SHFL_RT_LINK(pClient));
+#endif
+                    Assert(rc == VINF_SUCCESS || rc == VERR_FILE_NOT_FOUND || rc == VERR_PATH_NOT_FOUND);
+                }
+                else if (pszEnd == pszSrc)
+                    rc = VINF_SUCCESS;  /* trailing delimiter */
+                else
+                    rc = VERR_FILE_NOT_FOUND;
+
+                if (rc == VERR_FILE_NOT_FOUND || rc == VERR_PATH_NOT_FOUND)
+                {
+                    /* Path component is invalid; try to correct the casing. */
+                    rc = vbsfCorrectCasing(pClient, pszFullPath, pszSrc);
+                    if (RT_FAILURE(rc))
+                    {
+                        /* Failed, so don't bother trying any further components. */
+                        if (!fEndOfString)
+                            *pszEnd = RTPATH_DELIMITER; /* Restore the original full path. */
+                        break;
+                    }
+                }
+
+                /* Next (if any). */
+                if (fEndOfString)
+                    break;
+
+                *pszEnd = RTPATH_DELIMITER;
+                pszSrc = pszEnd + 1;
+            }
+            if (RT_FAILURE(rc))
+                Log(("Unable to find suitable component rc=%d\n", rc));
+        }
+        else
+            rc = VERR_FILE_NOT_FOUND;
+
+    }
+
+    /* Restore the final component if it was dropped. */
+    if (pszLastComponent)
+        *pszLastComponent = RTPATH_DELIMITER;
+
+    /* might be a new file so don't fail here! */
+    return VINF_SUCCESS;
+}
+
+
+/** @def VBSF_IS_PATH_SLASH
+ * Checks if @a a_ch is a path delimiter (slash, not volume spearator) for the
+ * guest or the host.
+ * @param   a_pClient   Pointer to the client data (SHFLCLIENTDATA).
+ * @param   a_ch        The character to inspect.
+ */
+#if 1
+# define VBSF_IS_PATH_SLASH(a_pClient, a_ch)     ( (a_ch) == '\\' || (a_ch) == '/' || RTPATH_IS_SLASH(a_ch) )
+#else
+# define VBSF_IS_PATH_SLASH(a_pClient, a_ch)     ( (RTUTF16)(a_ch) == (a_pClient)->PathDelimiter || RTPATH_IS_SLASH(a_ch) )
+#endif
+
+
+/**
+ * Check the given UTF-8 path for root escapes.
+ *
+ * Verify that the path is within the root directory of the mapping.  Count '..'
+ * and other path components and check that we do not go over the root.
+ *
+ * @returns VBox status code.
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_INVALID_NAME
+ *
+ * @param   pszPath         The (UTF-8) path to check.  Slashes has been convert
+ *                          to host slashes by this time.
+ *
+ * @remarks This function assumes that the path will be appended to the root
+ *          directory of the shared folder mapping.  Keep that in mind when
+ *          checking absolute paths!
+ */
+static int vbsfPathCheck(const char *pszPath)
+{
+    /*
+     * Walk the path, component by component and check for escapes.
      */
 
-    size_t i = 0;
     int cComponents = 0; /* How many normal path components. */
     int cParentDirs = 0; /* How many '..' components. */
 
     for (;;)
     {
-        /* Skip leading path delimiters. */
-        while (   i < cbPath
-               && (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/'))
-            i++;
+        char ch;
 
-        if (i >= cbPath)
-            break;
+        /* Skip leading path delimiters. */
+        do
+            ch = *pszPath++;
+        while (RTPATH_IS_SLASH(ch));
+        if (ch == '\0')
+            return VINF_SUCCESS;
 
         /* Check if that is a dot component. */
         int cDots = 0;
-        while (i < cbPath && pUtf8Path[i] == '.')
+        while (ch == '.')
         {
             cDots++;
-            i++;
+            ch = *pszPath++;
         }
 
-        if (   cDots >= 2 /* Consider all multidots sequences as a 'parent dir'. */
-            && (i >= cbPath || (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/')))
+        if (   cDots >= 1
+            && (ch == '\0' || RTPATH_IS_SLASH(ch)) )
         {
-            cParentDirs++;
-        }
-        else if (   cDots == 1
-                 && (i >= cbPath || (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/')))
-        {
-            /* Single dot, nothing changes. */
+            if (cDots >= 2) /* Consider all multidots sequences as a 'parent dir'. */
+            {
+                cParentDirs++;
+
+                /* Escaping? */
+                if (cParentDirs > cComponents)
+                    return VERR_INVALID_NAME;
+            }
+            /* else: Single dot, nothing changes. */
         }
         else
         {
-            /* Skip this component. */
-            while (   i < cbPath
-                   && (pUtf8Path[i] != '\\' && pUtf8Path[i] != '/'))
-                i++;
-
+            /* Not a dot component, skip to the end of it. */
+            while (ch != '\0' && !RTPATH_IS_SLASH(ch))
+                ch = *pszPath++;
             cComponents++;
         }
+        Assert(cComponents >= cParentDirs);
 
-        Assert(i >= cbPath || (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/'));
-
-        /* Verify counters for every component. */
-        if (cParentDirs > cComponents)
-        {
-            rc = VERR_INVALID_NAME;
-            break;
-        }
+        /* The end? */
+        Assert(ch == '\0' || RTPATH_IS_SLASH(ch));
+        if (ch == '\0')
+            return VINF_SUCCESS;
     }
-
-    return rc;
 }
 
 static int vbsfBuildFullPath(SHFLCLIENTDATA *pClient, SHFLROOT root, PSHFLSTRING pPath,
                              uint32_t cbPath, char **ppszFullPath, uint32_t *pcbFullPathRoot,
                              bool fWildCard = false, bool fPreserveLastComponent = false)
 {
-    int rc = VINF_SUCCESS;
-    char *pszFullPath = NULL;
-    size_t cbRoot;
+    /* Resolve the root prefix into a string. */
     const char *pszRoot = vbsfMappingsQueryHostRoot(root);
-
     if (   !pszRoot
-        || !(cbRoot = strlen(pszRoot)))
+        || !*pszRoot)
     {
         Log(("vbsfBuildFullPath: invalid root!\n"));
         return VERR_INVALID_PARAMETER;
     }
+    uint32_t cchRoot = (uint32_t)strlen(pszRoot);
+#if RTPATH_STYLE == RTPATH_STR_F_STYLE_DOS
+    Assert(!strchr(pszRoot, '/'));
+#endif
 
+    /*
+     * Combine the root prefix with the guest path into a full UTF-8 path in a
+     * buffer pointed to by pszFullPath.  cchRoot may be adjusted in the process.
+     */
+    int    rc;
+    size_t cchFullPath;
+    char  *pszFullPath = NULL;
     if (BIT_FLAG(pClient->fu32Flags, SHFL_CF_UTF8))
     {
-        /* Verify that the path is under the root directory. */
-        rc = vbsfPathCheck((const char *)&pPath->String.utf8[0], pPath->u16Length);
+        /* Strip leading slashes from the path the guest specified. */
+        uint32_t    cchSrc = pPath->u16Length;
+        const char *pszSrc = (char *)&pPath->String.utf8[0];
+        Log(("Root %s path %.*s\n", pszRoot, cchSrc, pszSrc));
 
-        if (RT_SUCCESS(rc))
+        while (   cchSrc > 0
+               && VBSF_IS_PATH_SLASH(pClient, *pszSrc))
         {
-            size_t cbUtf8FullPath = cbRoot + 1 + pPath->u16Length + 1;
-            char *utf8FullPath = (char *) RTMemAllocZ(cbUtf8FullPath);
+            pszSrc++;
+            cchSrc--;
+        }
 
-            if (!utf8FullPath)
+        /* Allocate a buffer that will be able to contain the root prefix and
+           the path specified by the guest. */
+        cchFullPath = cchRoot + cchSrc;
+        pszFullPath = (char *)RTMemAlloc(cchFullPath + 1 + 1);
+        if (RT_LIKELY(pszFullPath != NULL))
+        {
+            memcpy(pszFullPath, pszRoot, cchRoot);
+            if (!RTPATH_IS_SLASH(pszRoot[-1]))
             {
-                rc = VERR_NO_MEMORY;
-                *ppszFullPath = NULL;
-                Log(("RTMemAllocZ %x failed!!\n", cbUtf8FullPath));
+                pszFullPath[cchRoot++] = RTPATH_DELIMITER;
+                cchFullPath++;
             }
-            else
-            {
-                memcpy(utf8FullPath, pszRoot, cbRoot);
-                utf8FullPath[cbRoot] = '/';
-                memcpy(utf8FullPath + cbRoot + 1, &pPath->String.utf8[0], pPath->u16Length);
-                utf8FullPath[cbUtf8FullPath - 1] = 0;
-                pszFullPath = utf8FullPath;
 
-                if (pcbFullPathRoot)
-                    *pcbFullPathRoot = (uint32_t)cbRoot; /* Must index the path delimiter. */
-            }
+            if (cchSrc)
+                memcpy(&pszFullPath[cchRoot], pszSrc, cchSrc);
+
+            /* Terminate the string. */
+            pszFullPath[cchRoot + cchSrc] = '\0';
+            rc = VINF_SUCCESS;
         }
         else
         {
-            Log(("vbsfBuildFullPath: RTUtf16ToUtf8 failed with %Rrc\n", rc));
+            Log(("RTMemAlloc %x failed!!\n", cchFullPath + 1));
+            rc = VERR_NO_MEMORY;
         }
     }
-    else
+    else /* Client speaks UTF-16. */
     {
-#ifdef RT_OS_DARWIN
+#ifdef RT_OS_DARWIN /* Misplaced hack! See todo! */
 /** @todo This belongs in rtPathToNative or in the windows shared folder file system driver...
  * The question is simply whether the NFD normalization is actually applied on a (virtual) file
  * system level in darwin, or just by the user mode application libs. */
@@ -326,237 +552,125 @@ static int vbsfBuildFullPath(SHFLCLIENTDATA *pClient, SHFLROOT root, PSHFLSTRING
 
         CFRelease(inStr);
 #endif
-        /* Client sends us UCS2, so convert it to UTF8. */
-        Log(("Root %s path %.*ls\n", pszRoot, pPath->u16Length/sizeof(pPath->String.ucs2[0]), pPath->String.ucs2));
+
+        /* Strip leading slashes and calculate the UTF-8 length. */
+        size_t      cwcSrc  = pPath->u16Length / sizeof(RTUTF16);
+        PRTUTF16    pwszSrc = &pPath->String.ucs2[0];
+        Log(("Root %s path %.*ls\n", pszRoot, cwcSrc, pwszSrc));
+
+        while (   cwcSrc > 0
+               && *pwszSrc < 0x80
+               && VBSF_IS_PATH_SLASH(pClient, (char)*pwszSrc))
+        {
+            pwszSrc++;
+            cwcSrc--;
+        }
+
+        size_t      cchPathAsUtf8 = RTUtf16CalcUtf8Len(pwszSrc);
+#ifdef RT_OS_DARWIN
+        AssertReturnStmt(cchPathAsUtf8 >= cwcSrc, RTMemFree(pPath), VERR_INTERNAL_ERROR_3);
+#else
+        AssertReturn(cchPathAsUtf8 >= cwcSrc, VERR_INTERNAL_ERROR_3);
+#endif
 
         /* Allocate buffer that will be able to contain the root prefix and
-         * the pPath converted to UTF8. Expect a 2 bytes UCS2 to be converted
-         * to 8 bytes UTF8 in the worst case.
-         */
-        uint32_t cbFullPath = (cbRoot + ShflStringLength(pPath)) * 4;
-        pszFullPath = (char *)RTMemAllocZ(cbFullPath);
-        if (!pszFullPath)
+         * the pPath converted to UTF-8. */
+        cchFullPath = cchRoot + cchPathAsUtf8;
+        pszFullPath = (char *)RTMemAlloc(cchFullPath + 1 + 1);
+        if (RT_LIKELY(pszFullPath != NULL))
         {
-            rc = VERR_NO_MEMORY;
+            /* Copy the root prefix into the result buffer and make sure it
+               ends with a path separator. */
+            memcpy(pszFullPath, pszRoot, cchRoot);
+            if (!RTPATH_IS_SLASH(pszFullPath[cchRoot - 1]))
+            {
+                pszFullPath[cchRoot++] = RTPATH_DELIMITER;
+                cchFullPath++;
+            }
+
+            /* Append the path specified by the guest (if any). */
+            if (cchPathAsUtf8)
+            {
+                size_t cchActual;
+                char *pszDst = &pszFullPath[cchRoot];
+                rc = RTUtf16ToUtf8Ex(pwszSrc, cwcSrc, &pszDst, cchFullPath - cchRoot + 1, &cchActual);
+                AssertRC(rc);
+                AssertStmt(RT_FAILURE(rc) || cchActual == cchPathAsUtf8, rc = VERR_INTERNAL_ERROR_4);
+                Assert(strlen(pszDst) == cchPathAsUtf8);
+            }
+            else
+                rc = VINF_SUCCESS;
+
+            /* Terminate the string. */
+            pszFullPath[cchRoot + cchPathAsUtf8] = '\0';
         }
         else
         {
-            memcpy(pszFullPath, pszRoot, cbRoot + 1);
-            char *pszDst = pszFullPath;
-            size_t cbDst = strlen(pszDst);
-            size_t cb    = cbFullPath;
-            if (pszDst[cbDst - 1] != RTPATH_DELIMITER)
-            {
-                pszDst[cbDst] = RTPATH_DELIMITER;
-                cbDst++;
-            }
-
-            if (pcbFullPathRoot)
-                *pcbFullPathRoot = cbDst - 1; /* Must index the path delimiter.  */
-
-            pszDst += cbDst;
-            cb     -= cbDst;
-
-            if (pPath->u16Length)
-            {
-                /* Convert and copy components. */
-                PRTUTF16 pwszSrc = &pPath->String.ucs2[0];
-
-                /* Correct path delimiters */
-                if (pClient->PathDelimiter != RTPATH_DELIMITER)
-                {
-                    LogFlow(("Correct path delimiter in %ls\n", pwszSrc));
-                    while (*pwszSrc)
-                    {
-                        if (*pwszSrc == pClient->PathDelimiter)
-                            *pwszSrc = RTPATH_DELIMITER;
-                        pwszSrc++;
-                    }
-                    pwszSrc = &pPath->String.ucs2[0];
-                    LogFlow(("Corrected string %ls\n", pwszSrc));
-                }
-                if (*pwszSrc == RTPATH_DELIMITER)
-                    pwszSrc++;  /* we already appended a delimiter to the first part */
-
-                rc = RTUtf16ToUtf8Ex(pwszSrc, RTSTR_MAX, &pszDst, cb, NULL);
-                if (RT_FAILURE(rc))
-                {
-                    AssertFailed();
-#ifdef RT_OS_DARWIN
-                    RTMemFree(pPath);
-                    pPath = pPathParameter;
-#endif
-                    return rc;
-                }
-
-                cbDst = (uint32_t)strlen(pszDst);
-
-                /* Verify that the path is under the root directory. */
-                rc = vbsfPathCheck(pszDst, cbDst);
-                if (RT_FAILURE(rc))
-                {
-#ifdef RT_OS_DARWIN
-                    RTMemFree(pPath);
-                    pPath = pPathParameter;
-#endif
-                    return rc;
-                }
-
-                cb     -= cbDst;
-                pszDst += cbDst;
-
-                Assert(cb > 0);
-            }
-
-            /* Nul terminate the string */
-            *pszDst = 0;
+            Log(("RTMemAlloc %x failed!!\n", cchFullPath + 1));
+            rc = VERR_NO_MEMORY;
         }
+
 #ifdef RT_OS_DARWIN
         RTMemFree(pPath);
         pPath = pPathParameter;
 #endif
     }
-
     if (RT_SUCCESS(rc))
     {
-        /* When the host file system is case sensitive and the guest expects a case insensitive fs, then problems can occur */
-        if (     vbsfIsHostMappingCaseSensitive(root)
-            &&  !vbsfIsGuestMappingCaseSensitive(root))
+        Assert(strlen(pszFullPath) == cchFullPath);
+        Assert(RTPATH_IS_SLASH(pszFullPath[cchRoot - 1])); /* includes delimiter. */
+
+        if (pcbFullPathRoot)
+            *pcbFullPathRoot = cchRoot - 1; /* Must index the path delimiter. */
+
+        /*
+         * Convert guest path delimiters into host ones and check for attempts
+         * to escape the shared folder root directory.
+         *
+         * After this, there will only be RTPATH_DELIMITER slashes in the path!
+         *
+         * ASSUMES that the root path only has RTPATH_DELIMITER as well.
+         */
+        char  ch;
+        char *pszTmp = &pszFullPath[cchRoot];
+        while ((ch = *pszTmp) != '\0')
         {
-            RTFSOBJINFO info;
-            char *pszLastComponent = NULL;
-
-            if (fWildCard || fPreserveLastComponent)
-            {
-                /* strip off the last path component, that has to be preserved: contains the wildcard(s) or a 'rename' target. */
-                size_t cb = strlen(pszFullPath);
-                char *pszSrc = pszFullPath + cb - 1;
-
-                while (pszSrc > pszFullPath)
-                {
-                    if (*pszSrc == RTPATH_DELIMITER)
-                        break;
-                    pszSrc--;
-                }
-                if (*pszSrc == RTPATH_DELIMITER)
-                {
-                    bool fHaveWildcards = false;
-                    char *psz = pszSrc;
-
-                    while (*psz)
-                    {
-                        char ch = *psz;
-                        if (ch == '*' || ch == '?' || ch == '>' || ch == '<' || ch == '"')
-                        {
-                            fHaveWildcards = true;
-                            break;
-                        }
-                        psz++;
-                    }
-
-                    if (fHaveWildcards || fPreserveLastComponent)
-                    {
-                        pszLastComponent = pszSrc;
-                        *pszLastComponent = 0;
-                    }
-                }
-            }
-
-            /** @todo don't check when creating files or directories; waste of time */
-            rc = RTPathQueryInfoEx(pszFullPath, &info, RTFSOBJATTRADD_NOTHING, SHFL_RT_LINK(pClient));
-            if (rc == VERR_FILE_NOT_FOUND || rc == VERR_PATH_NOT_FOUND)
-            {
-                size_t cb = strlen(pszFullPath);
-                char   *pszSrc = pszFullPath + cb - 1;
-
-                Log(("Handle case insensitive guest fs on top of host case sensitive fs for %s\n", pszFullPath));
-
-                /* Find partial path that's valid */
-                while (pszSrc > pszFullPath)
-                {
-                    if (*pszSrc == RTPATH_DELIMITER)
-                    {
-                        *pszSrc = 0;
-                        rc = RTPathQueryInfoEx(pszFullPath, &info, RTFSOBJATTRADD_NOTHING, SHFL_RT_LINK(pClient));
-                        *pszSrc = RTPATH_DELIMITER;
-                        if (rc == VINF_SUCCESS)
-                        {
-#ifdef DEBUG
-                            *pszSrc = 0;
-                            Log(("Found valid partial path %s\n", pszFullPath));
-                            *pszSrc = RTPATH_DELIMITER;
-#endif
-                            break;
-                        }
-                    }
-
-                    pszSrc--;
-                }
-                Assert(*pszSrc == RTPATH_DELIMITER && RT_SUCCESS(rc));
-                if (    *pszSrc == RTPATH_DELIMITER
-                    &&  RT_SUCCESS(rc))
-                {
-                    pszSrc++;
-                    for (;;)
-                    {
-                        char *pszEnd = pszSrc;
-                        bool fEndOfString = true;
-
-                        while (*pszEnd)
-                        {
-                            if (*pszEnd == RTPATH_DELIMITER)
-                                break;
-                            pszEnd++;
-                        }
-
-                        if (*pszEnd == RTPATH_DELIMITER)
-                        {
-                            fEndOfString = false;
-                            *pszEnd = 0;
-                            rc = RTPathQueryInfoEx(pszSrc, &info, RTFSOBJATTRADD_NOTHING, SHFL_RT_LINK(pClient));
-                            Assert(rc == VINF_SUCCESS || rc == VERR_FILE_NOT_FOUND || rc == VERR_PATH_NOT_FOUND);
-                        }
-                        else if (pszEnd == pszSrc)
-                            rc = VINF_SUCCESS;  /* trailing delimiter */
-                        else
-                            rc = VERR_FILE_NOT_FOUND;
-
-                        if (rc == VERR_FILE_NOT_FOUND || rc == VERR_PATH_NOT_FOUND)
-                        {
-                            /* path component is invalid; try to correct the casing */
-                            rc = vbsfCorrectCasing(pClient, pszFullPath, pszSrc);
-                            if (RT_FAILURE(rc))
-                            {
-                                if (!fEndOfString)
-                                    *pszEnd = RTPATH_DELIMITER; /* restore the original full path */
-                                break;
-                            }
-                        }
-
-                        if (fEndOfString)
-                            break;
-
-                        *pszEnd = RTPATH_DELIMITER;
-                        pszSrc = pszEnd + 1;
-                    }
-                    if (RT_FAILURE(rc))
-                        Log(("Unable to find suitable component rc=%d\n", rc));
-                }
-                else
-                    rc = VERR_FILE_NOT_FOUND;
-
-            }
-            if (pszLastComponent)
-                *pszLastComponent = RTPATH_DELIMITER;
-
-            /* might be a new file so don't fail here! */
-            rc = VINF_SUCCESS;
+            if (VBSF_IS_PATH_SLASH(pClient, ch))
+                *pszTmp = RTPATH_DELIMITER;
+            pszTmp++;
         }
-        *ppszFullPath = pszFullPath;
+        LogFlow(("Corrected string %s\n", pszFullPath));
 
-        LogFlow(("vbsfBuildFullPath: %s rc=%Rrc\n", pszFullPath, rc));
+        rc = vbsfPathCheck(&pszFullPath[cchRoot]);
+        if (RT_SUCCESS(rc))
+        {
+            /*
+             * When the host file system is case sensitive and the guest expects
+             * a case insensitive fs, then problems can occur.
+             */
+            if (    vbsfIsHostMappingCaseSensitive(root)
+                && !vbsfIsGuestMappingCaseSensitive(root))
+                rc = vbsfCorrectPathCasing(pClient, pszFullPath, cchFullPath, fWildCard, fPreserveLastComponent);
+            if (RT_SUCCESS(rc))
+            {
+                /*
+                 * We're good.
+                 */
+                *ppszFullPath = pszFullPath;
+                LogFlow(("vbsfBuildFullPath: %s rc=%Rrc\n", pszFullPath, rc));
+                return rc;
+            }
+
+            /* Failed, clean up. */
+            Log(("vbsfBuildFullPath: %s rc=%Rrc\n", pszFullPath, rc));
+        }
+        else
+            Log(("vbsfBuildPath: Caught escape attempt: (%.*s) '%s'\n", cchRoot, pszFullPath, &pszFullPath[cchRoot]));
     }
 
+    if (pszFullPath)
+        RTMemFree(pszFullPath);
+    *ppszFullPath = NULL;
     return rc;
 }
 
