@@ -239,27 +239,72 @@ get_sysinfo()
         fi
         if test ! -z "$PKGFMRI"; then
             # The format is "pkg://solaris/system/kernel@0.5.11,5.11-0.161:20110315T070332Z"
+            #            or "pkg://solaris/system/kernel@5.12,5.11-5.12.0.0.0.4.1:20120908T030246Z"
             #            or "pkg://solaris/system/kernel@0.5.11,5.11-0.175.0.0.0.1.0:20111012T032837Z"
-            STR_KERN=`echo "$PKGFMRI" | sed 's/^.*\@//;s/\:.*//;s/.*,//'`
-            if test ! -z "$STR_KERN"; then
-                # The format is "5.11-0.161" or "5.11-0.175.0.0.0.1.0"
-                HOST_OS_MAJORVERSION=`echo "$STR_KERN" | cut -f1 -d'-'`
-                HOST_OS_MINORVERSION=`echo "$STR_KERN" | cut -f2 -d'-' | cut -f2 -d '.'`
+            #            or "pkg://solaris/system/kernel@5.12-5.12.0.0.0.9.1.3.0:20121012T032837Z"
+            STR_KERN_MAJOR=`echo "$PKGFMRI" | sed 's/^.*\@//;s/\,.*//'`
+            if test ! -z "$STR_KERN_MAJOR"; then
+                # The format is "0.5.11" or "5.12"
+                # Let us just hardcode these for now, instead of trying to do things more generically. It's not
+                # worth trying to bring more order to chaos as it's clear that the version numbering is subject to breakage
+                # as it has been seen in the past.
+                if test "$STR_KERN_MAJOR" = "5.12"; then
+                    HOST_OS_MAJORVERSION="12"
+                elif test "$STR_KERN_MAJOR" = "0.5.11" || test "$STR_KERN_MAJOR" = "5.11"; then
+                    HOST_OS_MAJORVERSION="11"
+                else
+                    # This could be the PSARC/2012/240 naming scheme for S12.
+                    # The format is "pkg://solaris/system/kernel@5.12-5.12.0.0.0.9.1.3.0:20121012T032837Z"
+                    # The "5.12" following the "@" is the nominal version which we ignore for now as it is
+                    # not set by most pkg(5) tools...
+                    # STR_KERN_MAJOR is now of the format "5.12-5.12.0.0.0.9.1.3.0:20121012T032837Z" with '9' representing
+                    # the build number.
+                    BRANCH_VERSION=STR_KERN_MAJOR
+                    HOST_OS_MAJORVERSION=`echo "$BRANCH_VERSION" | cut -f2 -d'-' | cut -f1,2 -d'.'`
+                    if test "HOST_OS_MAJORVERSION" = "5.12"; then
+                        HOST_OS_MINORVERSION=`echo "$BRANCH_VERSION" | cut -f2 -d'-' | cut -f6 -d'.'`
+                        return 0
+                    else
+                        errorprint "Failed to parse the Solaris kernel major version."
+                        exit 1
+                    fi
+                fi
+
+                # This applies only to S11 and S12 where the transitional "@5.12," component version is
+                # still part of the pkg(5) package FMRI. The regular S12 will follow the PSARC/2012/240 naming scheme above.
+                STR_KERN_MINOR=`echo "$PKGFMRI" | sed 's/^.*\@//;s/\:.*//;s/.*,//'`
+                if test ! -z "$STR_KERN_MINOR"; then
+                    # The HOST_OS_MINORVERSION is represented as follows:
+                    # For S12 it represents the build numbers. e.g. for 4  :  "5.11-5.12.0.0.0.4.1"
+                    # For S11 as the "nevada" version numbers. e.g. for 175:  "5.11-0.161" or "5.11-0.175.0.0.0.1.0"
+                    if test "$HOST_OS_MAJORVERSION" -eq 12; then
+                        HOST_OS_MINORVERSION=`echo "$STR_KERN_MINOR" | cut -f2 -d'-' | cut -f6 -d'.'`
+                    elif test "$HOST_OS_MAJORVERSION" -eq 11; then
+                        HOST_OS_MINORVERSION=`echo "$STR_KERN_MINOR" | cut -f2 -d'-' | cut -f2 -d'.'`
+                    else
+                        errorprint "Solaris kernel major version $HOST_OS_MAJORVERSION not supported."
+                        exit 1
+                    fi
+                else
+                    errorprint "Failed to parse the Solaris kernel minor version."
+                    exit 1
+                fi
             else
-                errorprint "Failed to parse the Solaris kernel version."
+                errorprint "Failed to parse the Solaris kernel package version."
                 exit 1
-            fi        
+            fi
         else
-            errorprint "Failed to detect the Solaris kernel version."
+            errorprint "Failed to detect the Solaris kernel package FMRI."
             exit 1
         fi
     else
         HOST_OS_MAJORVERSION=`uname -r`
         if test -z "$HOST_OS_MAJORVERSION" || test "$HOST_OS_MAJORVERSION" != "5.10";  then
-            # S11 without 'pkg' ?? Something's wrong... bail.
-            errorprint "Solaris $HOST_OS_MAJOR_VERSION detected without executable $BIN_PKG !? Confused."
+            # S11 without 'pkg'?? Something's wrong... bail.
+            errorprint "Solaris $HOST_OS_MAJORVERSION detected without executable $BIN_PKG !? I are confused."
             exit 1
         fi
+        HOST_OS_MAJORVERSION="10"
         if test "$REMOTEINST" -eq 0; then
             # Use uname to verify it's S10.
             # Major version is S10, Minor version is no longer relevant (or used), use uname -v so it gets something
@@ -276,7 +321,7 @@ get_sysinfo()
 
             REMOTE_S10=`$BIN_PKGCHK -l -p /kernel/amd64/genunix $BASEDIR_PKGOPT 2> /dev/null | grep SUNWckr | tr -d ' \t'`
             if test ! -z "$REMOTE_S10" && test "$REMOTE_S10" = "SUNWckr"; then
-                HOST_OS_MAJORVERSION="5.10"
+                HOST_OS_MAJORVERSION="10"
                 HOST_OS_MINORVERSION=""
             else
                 errorprint "Remote target $PKG_INSTALL_ROOT is not Solaris 10."
@@ -588,7 +633,7 @@ install_drivers()
         load_vboxbow
     else
         # If host is S10 or S11 (< snv_159) or vboxbow isn't shipped, then load vboxflt
-        if test "$HOST_OS_MAJORVERSION" = "5.10" || test "$HOST_OS_MINORVERSION" -lt 159 || test ! -f "$DIR_CONF/vboxbow.conf"; then
+        if test "$HOST_OS_MAJORVERSION" -eq 10 || (test "$HOST_OS_MAJORVERSION" -eq 11 && test "$HOST_OS_MINORVERSION" -lt 159) || test ! -f "$DIR_CONF/vboxbow.conf"; then
             load_vboxflt
         else
             # For S11 snv_159+ load vboxbow
@@ -597,9 +642,9 @@ install_drivers()
     fi
 
     # Load VBoxUSBMon, VBoxUSB
-    if test -f "$DIR_CONF/vboxusbmon.conf" && test "$HOST_OS_MAJORVERSION" != "5.10"; then
-        # For VirtualBox 3.1 the new USB code requires Nevada > 123
-        if test "$HOST_OS_MINORVERSION" -gt 123; then
+    if test -f "$DIR_CONF/vboxusbmon.conf" && test "$HOST_OS_MAJORVERSION" != "10"; then
+        # For VirtualBox 3.1 the new USB code requires Nevada > 123 i.e. S12+ or S11 b124+
+        if test "$HOST_OS_MAJORVERSION" -gt 11 || (test "$HOST_OS_MAJORVERSION" -eq 11 && test "$HOST_OS_MINORVERSION" -gt 123); then
             # Add a group "vboxuser" (8-character limit) for USB access.
             # All users which need host USB-passthrough support will have to be added to this group.
             groupadd vboxuser >/dev/null 2>&1
@@ -632,11 +677,7 @@ install_drivers()
                 load_module "drv/$MOD_VBOXUSB" "$DESC_VBOXUSB" "$FATALOP"
             fi
         else
-            if test -n "$HOST_OS_MINORVERSION"; then
-                warnprint "Solaris 5.11 build 124 or higher required for USB support. Skipped installing USB support."
-            else
-                warnprint "Failed to determine Solaris 5.11 snv version. Skipped installing USB support."
-            fi
+            warnprint "Solaris 11 build 124 or higher required for USB support. Skipped installing USB support."
         fi
     fi
 
@@ -880,8 +921,8 @@ postinstall()
 
             # plumb and configure vboxnet0 for non-remote installs
             if test "$REMOTEINST" -eq 0; then
-                # S11 175a renames vboxnet0 as 'netX', undo this and rename it back
-                if test "$HOST_OS_MAJORVERSION" = "5.11" && test "$HOST_OS_MINORVERSION" -gt 174; then
+                # S11 175a renames vboxnet0 as 'netX', undo this and rename it back (S12+ or S11 b175+)
+                if test "$HOST_OS_MAJORVERSION" -gt 11 || (test "$HOST_OS_MAJORVERSION" -eq 11 && test "$HOST_OS_MINORVERSION" -gt 174); then
                     vanityname=`dladm show-phys -po link,device | grep vboxnet0 | cut -f1 -d':'`
                     if test $? -eq 0 && test ! -z "$vanityname" && test "$vanityname" != "vboxnet0"; then
                         dladm rename-link "$vanityname" vboxnet0
