@@ -240,7 +240,8 @@ static int testLdrOne(const char *pszFilename)
                 for (unsigned iSym = 0; iSym < RT_ELEMENTS(aSyms); iSym++)
                 {
                     RTUINTPTR Value;
-                    int rc = RTLdrGetSymbolEx(aLoads[i].hLdrMod, aLoads[i].pvBits, aLoads[i].Addr, aSyms[iSym].pszName, &Value);
+                    int rc = RTLdrGetSymbolEx(aLoads[i].hLdrMod, aLoads[i].pvBits, aLoads[i].Addr,
+                                              UINT32_MAX, aSyms[iSym].pszName, &Value);
                     if (RT_SUCCESS(rc))
                     {
                         unsigned off = Value - aLoads[i].Addr;
