@@ -784,7 +784,7 @@ static DECLCALLBACK(void) hmR0InitIntelCpu(RTCPUID idCpu, void *pvUser1, void *p
         == (MSR_IA32_FEATURE_CONTROL_VMXON | MSR_IA32_FEATURE_CONTROL_LOCK))
         rc = VINF_SUCCESS;
     else
-        rc = VERR_VMX_MSR_LOCKED_OR_DISABLED;
+        rc = VERR_VMX_MSR_LOCKING_FAILED;
 
     hmR0FirstRcSetStatus(pFirstRc, rc);
 }
@@ -901,10 +901,9 @@ static DECLCALLBACK(void) hmR0EnableCpuCallback(RTCPUID idCpu, void *pvUser1, vo
  * @param   pvUser          The VM handle.
  * @param   pvUserIgnore    NULL, ignored.
  */
-static DECLCALLBACK(int32_t) hmR0EnableAllCpuOnce(void *pvUser, void *pvUserIgnore)
+static DECLCALLBACK(int32_t) hmR0EnableAllCpuOnce(void *pvUser)
 {
     PVM pVM = (PVM)pvUser;
-    NOREF(pvUserIgnore);
 
     /*
      * Indicate that we've initialized.
@@ -994,7 +993,7 @@ VMMR0DECL(int) HWACCMR0EnableAllCpus(PVM pVM)
     if (ASMAtomicReadBool(&g_HvmR0.fSuspended))
         return VERR_HWACCM_SUSPEND_PENDING;
 
-    return RTOnce(&g_HvmR0.EnableAllCpusOnce, hmR0EnableAllCpuOnce, pVM, NULL);
+    return RTOnce(&g_HvmR0.EnableAllCpusOnce, hmR0EnableAllCpuOnce, pVM);
 }
 
 
